
CREATE TABLE active_sessions (
  sid varchar(32) DEFAULT '',
  name varchar(32) DEFAULT '',
  val text,
  changed varchar(14) DEFAULT '' NOT NULL,
  PRIMARY KEY (sid,name)
);

CREATE TABLE mp3data (
	id		serial,
	tag		int4 NOT NULL,
	filename	varchar unique NOT NULL,
	size		int8 NOT NULL,
	artist		varchar,
	title		varchar,
	genre		varchar,
	album		varchar,
	comment		varchar,
	year		int8,
	lastmod		int8,
	verified	int4 NOT NULL DEFAULT '0',
	added		int8 NOT NULL,
	loc_id		int8 NOT NULL,
	track		int8,
	sec		int8,
	mpeg		varchar(10),
	layer		varchar(10),
	bitrate		int8,
	samplerate	int8,
	channel		varchar(30),
	hits		int4 default '0',
	available 	int4 NOT NULL DEFAULT '0',
	PRIMARY KEY (id)
);
CREATE UNIQUE INDEX fileid ON mp3data (filename, loc_id);

CREATE INDEX mp3_artist ON mp3data(artist);
CREATE INDEX mp3_genre ON mp3data(genre);
CREATE INDEX mp3_album ON mp3data(album);
CREATE INDEX mp3_year ON mp3data(year);



CREATE TABLE mp3location (
	id		serial,
	type		varchar(10) NOT NULL,
	name		varchar(255) NOT NULL,
	description	varchar(255),
	basedir		varchar(255) NOT NULL,
	last_update	timestamp DEFAULT '0000-01-01 00:00:00' NOT NULL,
	first_seen	timestamp DEFAULT '0000-01-01 00:00:00' NOT NULL,
	last_seen 	timestamp DEFAULT '0000-01-01 00:00:00' NOT NULL,
	seconds_online	int8 DEFAULT '0' NOT NULL,
	available	int4 DEFAULT '0' NOT NULL,
	PRIMARY KEY (id)
);


CREATE TABLE playlist (
	id		serial,
	name		varchar not null,
	owner		varchar,
	parent		int8 not null default '0',
	PRIMARY KEY (id),
        UNIQUE (name, owner)
);



CREATE TABLE playlist_detail (
	playlist_id int8 NOT NULL,
	sequence int4 NOT NULL,
	song_id int8,
	PRIMARY KEY (playlist_id, sequence)
);


CREATE TABLE users (
  aid varchar(32) DEFAULT '' NOT NULL,
  name varchar(50),
  url varchar(200),
  email varchar(50),
  pwd varchar(32),
  uid varchar(32) DEFAULT '' NOT NULL,
  perms varchar(255),
  PRIMARY KEY (aid)
);

CREATE UNIQUE INDEX k_uid ON users (uid);

CREATE TABLE clients (
	name		varchar(15) default '' not null,
	ip_pattern	char(15) default '' not null,
	priority	int4 default 0 not null,
	PRIMARY KEY (name)
);

CREATE TABLE network_mappings (
	loc_id		int8 default 0 not null,
	mount_point	varchar(255) default '' not null,
	clients 	varchar(255) default '' not null,
	is_default 	int2 default '' not null,
	PRIMARY KEY (loc_id, mount_point)
);

CREATE TABLE filters (
	id		serial,
	sys_name	varchar(50) not null,
	aid		varchar(32) not null,
	active		int4 not null,
	field		varchar(50) not null,
	relationship	varchar(20) not null,
	filter_value	varchar(100) not null,
	value_type	varchar(20) not null,
	display 	varchar(100) not null,
	PRIMARY KEY (id)
);

CREATE TABLE version (
	version varchar(10) not null
);


GRANT ALL ON users TO rimps;
GRANT ALL ON mp3data TO rimps;
GRANT ALL ON playlist TO rimps;
GRANT ALL ON playlist_detail TO rimps;
GRANT ALL ON version TO rimps;

