<?php 
//error_reporting(1);
// Include browsing options
require "layout/browse_header.inc";
require "lib/filter_functions.inc";
require "lib/random_sort.inc";

//Set default sort
if ($sort ==''){
	$sort = 'all';
}

//echo "<hr>\n";

if(empty($db))
	$db=new rimpsDB;

require "engine/load_playlist.inc";

////////////////////////////////////////////////////////////////////////
// Now build an appropriate query based on what we are browsing.  As we
// build the main query we will also build $count_query which will return
// only the number of rows which meet the criteria (so we can do paging)

 // IMPORTANT NOTE ON POSTGRESQL: 
 // PostgreSQL is a little whacked.  There isn't a query which will return a count of the
 // number of distinct combinations of album and artist (or at least I haven't been able to
 // figure one out.  So, we have to use the normal query without a limit statement.  We select
 // the data and then look at the number of unique rows.

$query = "SELECT";
$count_query = "SELECT count(";
// So far so good :)

//Depending on what we are browsing we will select different things

if ($browse == 'title'){
	// Select everything-- we may be displaying it
	$addQuery = " *";
	$query .= $addQuery;
	$count_query .= $addQuery;
}
elseif($browse == 'artist'){
	// We'll only need the artist, and each artist need be listed only even though
	// it is likely that each artist has several rows in the database
	$query .= " artist, count(*) as subtotal";
	$count_query .= " distinct artist";
	$groupBy = "artist";
}
elseif($browse == 'album'){
	// We'll need both the artist and the album
	$query .= " album, artist, count(*) as subtotal";
        if ($dbconfig['sqlserver'] == 'pgsql'){
           // See comment above "IMPORTANT NOTICE ABOUT POSTGRESQL
           $count_query .= ' *';
         }
         else{
                 $count_query .= " distinct album, artist";
         }
	$groupBy = 'album, artist';
}

// We also want to the the sum of the size when we count
$count_query .= " ) as matched, sum(size) as totalsize, sum(sec) as totaltime";

$addQuery = " FROM mp3data";
$query .= $addQuery;
$count_query .= $addQuery;

//echo "Query: $query<br>\n";

// If $index is set, then we have started drilling down...  We should include a where
// clause to limit our result set. 

if ($index){
	if ($sort == 'alpha'){
		// If we are using alpha keys, then index is the ascii value of the char
		$addQuery = " WHERE $browse like '" . chr($index) . "%'";
		$query .= $addQuery;
		$count_query .= $addQuery;
	}
	elseif ($sort == 'genre'){
		// In this case $index is the string of the genre
		$addQuery = " WHERE genre = '$index'";
		$query .= $addQuery;
		$count_query .= $addQuery;
	}
	elseif ($sort == 'year'){
		// $index is the year
		$addQuery = " WHERE year = $index";
		$query .= $addQuery;
		$count_query .= $addQuery;
	}
	elseif ($sort == 'added'){
		// $index is two values separated by a | character.  The first value represents
		// the time AFTER which we want to look for songs.  The second value (if present)
		// represents a time frame in seconds within which we should look
		
		$parts = explode('|', $index);
		$farEndPoint = $parts[0];
		$addQuery = " WHERE added > $farEndPoint";
		if ($parts[1] != ''){
			// Add in window
			$nearEndPoint = $farEndPoint + $parts[1];
			$addQuery .= " AND added < $nearEndPoint";
		}
		$query .= $addQuery;
		$count_query .= $addQuery;
	}
	elseif ($sort == 'popularity'){
		// $index is a range of hits separated by a | character.
		$parts = explode('|', $index);
		$near = $parts[0];
		$far = $parts[1];
		$addQuery = " WHERE hits BETWEEN $near AND $far";
		$query .= $addQuery;
		$count_query .= $addQuery;
	}
}
else{
	$addQuery = " WHERE 1=1";
	$query .= $addQuery;
	$count_query .= $addQuery;
}

// Process filters
$addQuery = processFilters($filters);
$query .= $addQuery;
$count_query .= $addQuery;


// This variable is used later in pager.inc to allow a user to add all matching songs to
// a playlist.  Additionally we put it in the session
// because it is a query that gets passed to a future page and it shouldn't be tampered
// with.  To further be safe, we pass only the stuff after FROM mp3data...  This makes it
// much more difficult to get in a malicious query.  Right before we perform the query
// We'll add the select back on.

$totalResultSetQuery = stristr($query, 'FROM');
//echo "totalResultSetQuery set to $totalResultSetQuery";

// Add group by
if ($groupBy && $dbconfig["sqlserver"]=="mysql"){
	$query .= " GROUP BY $groupBy";
}

// We always want to order by the item we are browsing (unless it is random)
if ($sort == "random" && $dbconfig["sqlserver"] == "pgsql"){
	$query = $query . random_sort($query); }
elseif ($sort == "random"){
	$query .= " ORDER by rand()";
}

// Add group by
if ($groupBy && $dbconfig["sqlserver"]!="mysql"){
	$query .= " GROUP BY $groupBy";
}

// $page is used to track which page we are on, if the result set is large enough
// that it won't fit on one page
if((int)$page<=1)
	$page=1;

// Limit the number of rows selected at a time
if ($dbconfig["sqlserver"] == "mysql") {
	$query.=" LIMIT ".(($page-1)*$config["num_per_page"]).",".($config["num_per_page"]);
	} elseif ($dbconfig["sqlserver"] == "pgsql") {
	$query.=" LIMIT ".($config["num_per_page"]).",".(($page-1)*$config["num_per_page"]);
	}

//print "browse=$browse<br>\n";
//print "sort=$sort<br>\n";
//print "<br>query=$query<br>\n";


// Should we display entries from mp3data?  If index is set, or we are sorting by 
// all or random, then YES.  If not, then the queries we built above we don't need
// to run yet.  


if ($index || $sort=='all' || $sort=='random') {
	
	// Find out how many songs meet the given criteria:
	$db->query($count_query);
	if ($db->num_rows() > 0){
		$db->next_record();
		$matched = $db->f('matched');
		$total = $db->f('totalsize');
		$time = $db->f('totaltime');
	}
	
	// Display pager at the top
	include ("layout/pager.inc");
	
	//if($sort!='random' && $sort!= 'all')
	//	echo "<hr>\n";
	
	//print "query=$query<br>\n";

	// Finally do the query and display the results.
	
	msg("Browse query: $query");

	$db->query ($query);

	include("layout/show_music.inc");
}	

/////////////////////////////////////////////////////////////////////////////
// Display the pager.  It will have how many songs were found and how many
// megabytes that represents as well as the current and available pages

	
if($matched>0){
	include "layout/pager.inc";
}


// Show the browse palette which lists the years, genres, etc. when browsing
require 'layout/browse_palette.inc';


echo "<hr>\n";

?>
