<?
// Allows adding and editing of filters

require 'lib/filter_functions.inc';


if ($edit_filter){
	$title = 'Edit Filter';
}
else{
	$title = 'Add Filter';
}

if (empty($db)){
	$db = new rimpsDB;
}

// Build an array of the fields which can be customized
$availFields['genre'] = 'genre';
$availFields['artist'] = 'artist';
$availFields['size'] = 'size';
$availFields['sec'] = 'play time';
$availFields['title'] = 'title';
$availFields['album'] = 'album';
$availFields['comment'] = 'comment';
$availFields['year'] = 'year';
$availFields['lastmod'] = 'time file last modified';
$availFields['added'] = 'time added';
$availFields['loc_id'] = 'location ID';
$availFields['track'] = 'track';
$availFields['mpeg'] = 'MPEG version';
$availFields['layer'] = 'MPEG layer';
$availFields['samplerate'] = 'sample rate';
$availFields['channel'] = 'stereo channels';
$availFields['hits'] = 'times song played';
$availFields['available'] = 'location available';

// Build an array to show data types
$dataType['genre'] = 'string';
$dataType['artist'] = 'string';
$dataType['size'] = 'number';
$dataType['sec'] = 'number';
$dataType['title'] = 'string';
$dataType['album'] = 'string';
$dataType['comment'] = 'string';
$dataType['year'] = 'number';
$dataType['lastmod'] = 'number';
$dataType['added'] = 'number';
$dataType['loc_id'] = 'number';
$dataType['track'] = 'number';
$dataType['mpeg'] = 'string';
$dataType['layer'] = 'string';
$dataType['samplerate'] = 'string';
$dataType['channel'] = 'string';
$dataType['hits'] = 'number';
$dataType['available'] = 'number';

// Build an array of relationships
$availRelationship['contains'] = 'LIKE';
$availRelationship['does not contain'] = 'NOT LIKE';
$availRelationship['is exactly'] = '=';
$availRelationship['is not'] = '!=';
$availRelationship['is greater than'] = '>';
$availRelationship['is less than'] = '<';

// Build an array of types and associate with relationships
$availTypes['string'] = 'is exactly,is not,contains,does not contain';
$availTypes['number'] = 'is exactly,is not,is greater than,is less than';

function validateFilterSettings(){
	global $field, $relationship, $filter_value, $dataType;
	if($field == ''){
		rimps_error("You must choose a the field which this filter will act against");
		return 0;
	}
	elseif($relationship ==''){
		rimps_error("Choose a relationship:  Is exactly, is not, etc.");
		return 0;
	}
	elseif($filter_value== ''){
		rimps_error("Enter a value to filter against");
		return 0;
	}
	elseif($dataType[$field] == 'number' && !ereg('[[:digit:]\.]+', $filter_value)){
		rimps_error("The value to filter against $field must be a number.  Please enter a number.");
		return 0;
	}
	else{
		return 1;
	}
}


// Process adding filters
if ($filterAction == 'add'){
	if (validateFilterSettings()){		
		// All checks out fine.  Add new filter
		if ($display == ''){
			$display = "Show only files where ".$availFields[$field]." ".$relationship." ".$filter_value;
		}
		if ($relationship == 'contains' || $relationship == 'does not contain'){
			$filter_value = "%$filter_value%";
		}
		if ($dataType[$field] == 'string'){
			$filter_value = "'$filter_value'";
		}
		$sql = "INSERT INTO filters (aid, active, field, relationship, filter_value, ".
			"value_type, display, sys_name) VALUES ('".$auth->auth['uname'].
			"', 1, '$field', '".$availRelationship[$relationship]."', $filter_value, '".
			$dataType[$field]."', '$display', '')";
			
				/*
				id int not null PRIMARY KEY auto_increment,
				sys_name varchar(50) not null,
				aid varchar(32) not null, 
				active tinyint not null,
				field varchar(50) not null,
				relationship varchar(20) not null,
				filter_value varchar(100) not null,
				value_type varchar(20) not null,
				display varchar(100) not null
	
				*/
	
		msg ("query: $sql");
		$db->query($sql);
		rimps_msg("Added filter");
		// Refresh active filters
		$filters = getActiveFilters();
		$action='filter';
		include('engine/filter.inc');
		page_close();
		exit;		
	}
}
elseif($filterAction != ''){
	// Process editing filter
	
	if (validateFilterSettings()){
		if ($display == ''){
			$display = "Show only files where ".$availFields[$field]." ".$relationship." ".$filter_value;
		}
		if ($relationship == 'contains' || $relationship == 'does not contain'){
			$filter_value = "%$filter_value%";
		}
		if ($dataType[$field] == 'string'){
			$filter_value = "'$filter_value'";
		}
		$sql = "UPDATE filters SET field = '$field', relationship = '".$availRelationship[$relationship]."', filter_value = $filter_value, value_type = '".$dataType[$field]."', display = '$display' ".
			"WHERE aid = '".$auth->auth['uname']."' AND sys_name = '' AND id = $filterAction";
		msg ("Update filter query: $sql");
		$db->query($sql);		
		$filters = getActiveFilters();
		rimps_msg("Filter updated");
		$action = 'filter';
		include('engine/filter.inc');
		page_close();
		exit;
	}
}
elseif($edit_filter != ''){
	// Load up filter defaults for editing
	$sql = "SELECT * FROM filters where aid = '".$auth->auth['uname']."' AND id = $edit_filter";
	$db->query($sql);
	while($db->next_record()){
		$field = $db->f('field');
		$filter_value = $db->f('filter_value');
		$filter_value = str_replace('%', '', $filter_value);
		$filter_value = str_replace("'", '', $filter_value);
		$display = $db->f('display');
		reset($availRelationship);
		while(list($value, $text) = each($availRelationship)){
			if ($text == $db->f('relationship')){
				$relationship = $value;
				msg("Set relationship to: $relationship");
			}
		}
	}
}


// Javascript to dynamically change relationship drop down
?>
<script language=javascript>

function setRelationship(myForm, rel) {
	for (var i = 0; i < myForm.relationship.length; i++){
		if (myForm.relationship.options[i].text == rel){
			myForm.relationship.options[i].selected=true
		}   
	}
}

function resetSelect(selectBox){
	var len = selectBox.length;
	while(selectBox.length > 0){
		selectBox.options[0] = null;
	}
	
}

function getRelationshipOptions(varType){
	var options = new Array;
	<?
	reset($availTypes);
	while(list($type, $list) = each($availTypes)){
		echo 'if (varType == "'.$type.'"){'."\n";
		$types = explode(',', $list);
		reset($types);
		$i=0;
		while(list($tmp, $type) = each($types)){
			echo "options[$i] = new Option(\"$type\");\n";
			//echo "alert('Setting option to $type');\n";
			$i++;
		}
		echo "}\n";
	}
	?>
	return options;
}

function getVarType(name){
	//alert ("Name: "+name);
	var value = null;
	<?
	reset($availFields);
	while(list($fieldName, $tmp) = each($availFields)){
		echo 'if (name == "'.$fieldName.'"){';
		echo 'value = "'.$dataType[$fieldName].'"';
		echo '}';
	}
	?>
	return value;
}

function changeRelationships(inForm){
	var selected = inForm.field.selectedIndex;
	//alert("selected: "+selected);
	var varType = getVarType(inForm.field.options[selected].value);
	//alert("vartype: "+varType);
	resetSelect(inForm.relationship);
	var options = getRelationshipOptions(varType);
	for(var i=0; i<options.length; i++){
		inForm.relationship.options[i] = options[i];
	}
}

function updateDisplay(inForm){
	if (inForm.autoSetDisplay.value == 0){
		return;
	}
	var text = "Show only files where ";
	text = text + inForm.field.options[inForm.field.selectedIndex].text + " ";
	text = text + inForm.relationship.options[inForm.relationship.selectedIndex].text + " ";
	text = text + inForm.filter_value.value;
	inForm.display.value = text;
}
</script>

<?$theme->edge_title($title, $config['site_width']/2);?>
<form action="<?echo $config['baseurl']?>" name=filterForm>
<table border=0 width=100%>
<tr><td>
Show only files where
</td></tr>
<tr><td align=right><nobr>
<select name=field onChange="changeRelationships(this.form);updateDisplay(this.form)">
<option>
<?reset($availFields);
while(list($fieldName, $type) = each($availFields)){
	echo "<option value=\"$fieldName\"";
	if ($fieldName == $field){
		echo " SELECTED ";
	}
	echo ">".$availFields[$fieldName]."\n";
	
}?>
</select>
<select name=relationship onChange="updateDisplay(this.form)">
<?reset($availRelationship);
while(list($fieldName, $type) = each($availRelationship)){
	echo "<option";
	if ($fieldName == $relationship){
		echo " SELECTED ";
	}
	echo ">$fieldName\n";
}?>
</select></nobr>
</td></tr>
<tr><td align=right>
<input type=text name=filter_value value="<?echo $filter_value?>" size=30 maxlength=100 onChange="updateDisplay(this.form)">
</td></tr>
<tr><td><p>
Filter description (optional):<br>
<input type=text name=display value="<?echo $display?>" size=40 maxlength=100>

</td></tr>
<tr><td align=right>
<input type=submit value="<?echo $title?>">
</td></tr>
</table>
<input type=hidden name=action value="<?echo $action?>">
<?
// If we already have a display value 
if ($edit_filter==''){
	$autoSet = 1;
}
else{
	$autoSet =0;
}?>
<input type=hidden name=autoSetDisplay value=<?echo $autoSet?>>
<input type=hidden name=edit_filter value="<?echo $edit_filter?>">
<?if ($edit_filter ==''){?>
<input type=hidden name=filterAction value=add>
<?}else{?>
<input type=hidden name=filterAction value="<?echo $edit_filter?>">
<?}?>
</form>
<?
$theme->edge_title_end();
?>
<script language=javaScript>
changeRelationships(document.filterForm);
<?if ($edit_filter != ''){
	// Set initial value for relationship drop-down if editing
	echo "setRelationship(document.filterForm, '$relationship');";
}
?>
</script>