<?
// Upgrade engine.  This script will look at the version of RIMPS that was previously running
// and then try to find an upgrade path.  It will look in the directory upgrade_paths for a directory
// named after the previous version.  It will then look for a sub directory of this directory named
// after the version that has been installed.  If this directory is found, the upgrade will execute.  If
// this directory is NOT found, the user will be presented with a list of alternative upgrades (this
// would allow a user to upgrade from one version to the next and then to yet another).

// In the target directory for upgrade paths (for example upgrading from version 1.0 to version 3.0 the
// directory would be upgrade_paths/1.0/3.0) the file upgrade_script is loaded and executed.  There should
// also be SQL statements to be executed in the format filename.database (for example sql1.mysql or
// sql1.pgsql).

// upgrade.inc should perform all the necessary options.  We are going to be optimistic and assume that
// the upgrade is going to be successful.  If this is not the case, we need to set the $successfulUpgrade
// global to 0.  Finally, setting $successfulUpgrade to -1 means that the upgrade process is not complete.
// Likely we are asking for more information from the user and should therefore NOT complete the process.

if ($rimpsVersion == 'undef' || $rimpsVersion ==''){

	// $rimpsVersion will be undef if the version table doesn't exist.  This can mean one of two
	// things.  Either this is a brand new install, OR this is verison 1.0.  To test we'll look at
	// the tables in the RIMPS database.  If they exist, this is verison 1.0.  If they don't
	// exist this this is a new install.

	if($dbconfig["sqlserver"]=="mysql")
		$query="SHOW TABLES";
	elseif($dbconfig["sqlserver"]=="pgsql")
		$query="SELECT tablename FROM pg_tables WHERE tablename NOT LIKE 'pg_%'";
	else
		die("</td></tr></table><h2>Sorry unknown database type</h2>\n");

	$db->query($query);

	if ($db->num_rows()>0){
		$rimpsVersion = '1.0';
	}
	else{
		$rimpsVersion = 'NEW';
	}
}

function doSQL($name){
	// Processes a file full of SQL commands.  It takes as an argument the name of the SQL upgrade
	// file.  It then appends the database platform after a dot.  If the file exists, it will
	// process each line of the file and do the database commands.  
	// If all is successful it returns a 1, otherwise returns 0.
	// Also note that if it fails it sets successfulUpgrade to 0.
	
	// Check to see if the file exists
	$file = $GLOBALS['upgradePath']."/".$name.'.'.$GLOBALS['dbconfig']['sqlserver'];
	echo "Attempting to process SQL command file $file<br>";
	if (!file_exists($file)){
		echo "Error: the database upgrade file for ".$GLOBALS['dbconfig']['sqlserver']." does not exist";
		return 0;
	}
	echo "Processing SQL command file $file<br>";
	$db = new rimpsDB;
	$db->Halt_On_Error = 'no';
	$filelines = file($file);
	$file = implode('',$filelines);
	$sqlCommands = explode(';', $file);
	reset($sqlCommands);
	while(list($tmp, $command) = each ($sqlCommands)){
		$command=trim($command);
		//mysql doesn't like to process comments through PHP so kill them.
		$command=ereg_replace("#[^\n]*\n","",$command);
		if (!empty($command)){
			$db->query($command);
			if ($db->Errno != 0 || $db->Error != ''){
				echo "The SQL command - $command - failed with error: ".$db->Error."(".$db->Errno.")<br>";
				$GLOBALS['successfulUpgrade']=0;
				return 0;
			}
		}
	}
	return 1;
}

$successfulUpgrade=1;

?>

<b>Installation of a new version of RIMPS has been detected!</b><br>
You have been running version <?echo $rimpsVersion?>.<br>  Version <?echo RIMPS_VERSION?> has just been installed.<p>
<?flush()?>
Detecting upgrade options...<br>
<?
$upgradePath = "upgrade_paths/$rimpsVersion";
if (is_dir($upgradePath)){
	$upgradeOptionsDir = dir($upgradePath);
	while($dir = $upgradeOptionsDir->read()){
		if ($dir != '.' && $dir != '..'){
			if (is_dir($upgradePath."/".$dir)){
				$upgradeOptions[] = $dir;
			}
		}
	}
	$upgradeOptionsDir->close();
	if (sizeof($upgradeOptions) > 0){
		echo sizeof($upgradeOptions)." potential upgrade options found.<br>";
		// Look for an exact match
		reset ($upgradeOptions);
		while (list($tmp, $option) = each ($upgradeOptions)){
			if ($option == RIMPS_VERSION){
				$selectedUpgrade = $option;
				break;
			}
		}
		if ($selectedUpgrade == ''){
			// Give the user the available options
			echo "While no upgrade path from version $rimpsVersion to ".RIMPS_VERSION." was found, other options are available.  Please choose one from the list below:<br>";
			echo "<ul>";
			reset ($upgradeOptions);
			while (list($tmp, $option) = each ($upgradeOptions)){
				echo "<li><a href=".$config['base_url']."?selectedUpgrade=$option>$option</a>";
			}
			echo "</ul>";
		}
		else{
			echo "<br><b>An upgrade path from version $rimpsVersion to $selectedUpgrade was found.  Executing...</b><br>";
			$upgradePath = $upgradePath."/".$selectedUpgrade;
			if (file_exists($upgradePath."/upgrade.inc")){
				
			}
			else{
				$successfulUpgrade=0;
				echo "Error: The upgrade file for $rimpsVersion to $selectedUpgrade does not exist.  This usually means that the new version was not installed correctly.  Please verify the installation and try again.";
			}
			
			// OK everything is SET!   Hand over the upgrade process to upgrade.inc
			include($upgradePath."/upgrade.inc");
		}
	}
	else{
		echo "Error: There are no upgrade paths for version $rimpsVersion found<br>";
		$successfulUpgrade=0;
	}
}
else{
	echo "Error: No upgrade paths for version $rimpsVersion found<br>";
	$successfulUpgrade=0;
}

echo "<br><br>";
if ($successfulUpgrade==1){
	$sql = "DELETE FROM version";
	$db->query($sql);
	$sql = "INSERT INTO version (version) VALUES ('".RIMPS_VERSION."')";
	$db->query($sql);
	echo "RIMPS successfully upgraded from version $rimpsVersion to ".RIMPS_VERSION."<p>";
	echo "<a href=".$config['base_url'].">Continue</a><br>";
}
elseif($successfulUpgrade==0){
	echo "The upgrade of RIMPS version $rimpsVersion to ".RIMPS_VERSION." <b>failed</b>.<br>";
	echo "Please review the messages above to diagnose the problem and correct it if possible.  ";
	echo "If you are unable to correct the problem, you will need to re-install version $rimpsVersion";
	echo " to continue to use RIMPS.<p>";
}
?>