<?
// Functions to manipulate the mp3 data in the mp3data table

function prepareForDB($values, $fields){
	// Returns an array containing name values pairs ready to be inserted or updated
	// into a database.  The data for the name values pairs is based on the $values
	// array.  The $fields array lists the fields in the database as the keys and has
	// either 'number' or 'string' as the value, depending on the type of the database
	// field.

	reset($values);
	while(list($key, $value) = each($values)){
		if ($fields[$key]){
			if ($fields[$key]=='number'){
				// Force to number
				$value = $value + 0;
				if ($value == ''){
					// If there isn't something in this variable the insert will
					// fail because one value will be missing...
					$value = 0;
				}
				$v[$key] = $value;
			}
			else{
				$v[$key] = "'".addslashes(stripslashes($value))."'";
			}
		}
	}

	return $v;
}

function mp3dataInsertQuery($mp3data, $fields){
	// returns an insert query for mp3data using $mp3data as the values
	// and $fields to describe if the values are strings or numbers
	
	// Since we are adding this to the database, we want to record the date
	$mp3data['added'] = time();
	
	$q = prepareForDB($mp3data, $fields);

	// Build the actual query
	reset($q);
	while(list($key, $value) = each($q)){
		// Build a list of field names and values for the query
		$values[] = $value;
		$keys[] = $key;
	}

	$sqlKeys = implode(',',$keys);
	$sqlValues = implode(',', $values);
	$sql = "INSERT INTO mp3data ($sqlKeys) VALUES ($sqlValues)";
	return $sql;
}

function mp3dataUpdateQuery($mp3data, $fields){
	// returns an update query for mp3data using $mp3data as the values
	// and $fields to describe if the values are strings or numbers
	
	$q = prepareForDB($mp3data, $fields);

	// Build a list of $key = $value;
	reset($q);
	while(list($key, $value) = each($q)){
		$pairs[] = "$key = $value";
	}
	$sqlPairs = implode(',', $pairs);
	$sql = "UPDATE mp3data SET $sqlPairs WHERE filename = '".addslashes($mp3data['filename'])."'";
	return $sql;
}

function commitSongToDB($file, $codec='mpeg', $locationID='') {
	// Define the fields in the mp3data table.  Returns 'updated' if the song was updated
	// 'added' if the song was added, 'nothing' if nothing was done and 'failed' if the commit
	// fails.

	// Describe the database fields as strings or numbers ('string'/'number');
	$mp3dataField['title'] = 'string';
	$mp3dataField['filename'] = 'string';
	$mp3dataField['size'] = 'number';
	$mp3dataField['artist'] = 'string';
	$mp3dataField['genre'] = 'string';
	$mp3dataField['album'] = 'string';
	$mp3dataField['comment'] = 'string';
	$mp3dataField['year'] = 'number';
	$mp3dataField['lastmod'] = 'number';
	$mp3dataField['tag'] = 'number';
	$mp3dataField['verified'] = 'number';
	$mp3dataField['track'] = 'number';
	$mp3dataField['sec'] = 'number';
	$mp3dataField['mpeg'] = 'string';
	$mp3dataField['layer'] = 'string';
	$mp3dataField['bitrate'] = 'number';
	$mp3dataField['samplerate'] = 'number';
	$mp3dataField['channel'] = 'string';
	$mp3dataField['added'] = 'number';
	$mp3dataField['available'] = 'number';
	
	if ($locationID!=''){
		$mp3dataField['loc_id'] = 'number';
	}

	$db = new rimpsDB;
	$lastMod = filemtime($file);

	// Check to see if this file is already in the database
	$sql = "SELECT lastmod FROM mp3data WHERE filename = '".addslashes($file)."'";
	$db->query($sql);

	// Optimization:  If this file already exists in the database and it hasn't changed
	// since last added, we can just marked it as "verified" and cut loose.  This way we
	// don't have to read the ID3 tag for the file.
	if ($db->num_rows()>0){
		$db->next_record();
		if ($lastMod == $db->f('lastmod')){
			// No change for this song
			$sql = "UPDATE mp3data SET verified=1 WHERE filename = '".addslashes($file)."'";
			$db->query($sql);
			return 'nothing';
		}
	}

	if ($codec == 'mpeg')
		$mp3data = readID3($file);
	elseif ($codec == 'vorbis')
		$mp3data = readVorbis($file);
	else
		die("oops");
	$mp3data['lastmod'] = $lastMod;
	$mp3data['filename'] = $file;
	$mp3data['tag'] = 1;
	$mp3data['verified'] = 1;
	$mp3data['loc_id'] = $locationID;
	$mp3data['added'] = time();
	$mp3data['available'] = 1;
	
	if ($db->num_rows()>0){
		$db->next_record();
		// This song exists-- update the database
		msg("song changed: $file");
		$sql = mp3dataUpdateQuery($mp3data, $mp3dataField);
		$addedOrUpdated = 'updated';
	}
	else{
		// This is a new song-- add it to the database
		msg("new song: $file");
		$sql = mp3dataInsertQuery($mp3data, $mp3dataField);
		$addedOrUpdated = 'added';
		// Keep track of the files added because later we will want to see
		// if they represent files already in the database...
		if ($GLOBALS['config']['friendly_moves'] == 'yes'){
			$GLOBALS['fileAdded'][] = $mp3data;
		}
	}
	msg("Performing query: $sql");
	$result = $db->query($sql);
	if (!$result){
		$addedOrUpdated = 'failed';
	}
	return $addedOrUpdated;
}

function removeSongFromDB($id){
	// Removes a song from mp3data and all playlists which reference it.
	$db = new rimpsDB;
	$sql = "DELETE FROM mp3data WHERE id = $id";
	$db->query($sql);
	$sql = "DELETE FROM playlist_detail WHERE song_id = $id";
	$db->query($sql);
}
?>
