<?php
// The MP3Info class tries to get as much information from a
// mp3 as possible.
// uses id3v1.1 tag, header, Xing header and filename
// to get information.
class MP3Info
{
	// mp3 information readonly, please.
	var $filename="";
	var $filesize=0;
	var $artist="";
	var $title="";
	var $genre="";
	var $album="";
	var $track=0;
	var $comment="";
	var $year=0;
	var $sec=0;
	// header fields (look at mpeg layer 3 file spec for meanings)
	var $mpeg=""; 			// mpeg version
	var $layer="";			// layer type
	var $vbr=0;
	var $bitrate=0;
	var $samplerate=0;
	var $channel="";	// stereo, joint stereo, dual channel(stereo), mono
	var $copyright=0;
	var $original=0;
	var $emphasis="";		// emphasis
	var $crc="";			// crc
	var $tag=0;

	var $errormsg;			// contains an errormessage, set by read
	
	// private:
	var $fh;
    // Set up an array of genre names
    	// Previous list of genres was incorrect.  This is the corrected listing.
	var $genrename = array(
				"Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk",
				"Grunge", "Hip-Hop", "Jazz", "Metal", "New Age", "Oldies", "Other",
				"Pop", "R&B", "Rap", "Reggae", "Rock", "Techno",
				"Industrial", "Alternative", "Ska", "Death Metal", "Pranks",
				"Soundtrack", "Euro-Techno", "Ambient", "Trip-Hop", "Vocal",
				"Jazz+Funk", "Fusion", "Trance", "Classical", "Instrumental",
				"Acid", "House", "Game", "Sound Clip", "Gospel", "Noise",
				"AlternRock", "Bass", "Soul", "Punk", "Space", "Meditative",
				"Instrumental Pop", "Instrumental Rock", "Ethnic", "Gothic",
				"Darkwave", "Techno-Industrial", "Electronic", "Pop-Folk",
				"Eurodance", "Dream", "Southern Rock", "Comedy", "Cult", "Gangsta",
				"Top 40", "Christian Rap", "Pop/Funk", "Jungle", "Native American",
				"Cabaret", "New Wave", "Psychedelic", "Rave", "Showtunes", "Trailer",
				"Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz", "Polka", "Retro",
				"Musical", "Rock & Roll", "Hard Rock", "Folk", "Folk/Rock",
				"National Folk", "Swing", "Fast-Fusion", "Bebob", "Latin", 
				"Revival", "Celtic", "Bluegrass", "Avantgarde", "Gothic Rock", 
				"Progressive Rock", "Psychedelic Rock", "Symphonic Rock", 
				"Slow Rock", "Big Band", "Chorus", "Easy Listening", 
				"Acoustic", "Humour", "Speech", "Chanson", "Opera", 
				"Chamber Music", "Sonata", "Symphony", "Booty Bass",
				"Primus", "Porn Groove", "Satire", "Slow Jam", "Club", "Tango",
				"Samba", "Folklore", "Ballad", "Power Ballad", "Rhythmic Soul",
				"Freestyle", "Duet", "Punk Rock", "Drum Solo", "A capella",
				"Euro-House", "Dance Hall", "Goa", "Drum & Bass", "Club-House",
				"Hardcore", "Terror", "Indie", "BritPop", "Negerpunk", "Polsk Punk",
				"Beat", "Christian Gangsta Rap", "Heavy Metal", "Black Metal", "Crossover",
				"Contemporary Christian", "Christian Rock", "Merengue", "Salsa", 
				"Trash Metal", "Anime", "Jpop", "Synthpop"
				);

	var $mpeg_tbl = array("MPEG 2.5","UNKNOWN!","MPEG 2","MPEG 1");
	var $layer_tbl = array("UNKNOWN!","Layer III","Layer II","Layer I");
	var $bitrate_tbl =
	   array("MPEG 1"=>array("Layer I"=> array(0,32,64,96,128,160,192,224,256,288,320,352,384,416,448,0),
							 "Layer II" => array(0,32,48,56,64,80,96,112,128,160,192,224,256,320,384,0),
							 "Layer III"=> array(0,32,40,48,56,64,80,96,112,128,160,192,224,256,320,0)),
			 "MPEG 2"=>array("Layer I"  => array(0,32,48,56,64,80,96,112,128,144,160,176,192,224,256,0),
							 "Layer II" => array(0,8,16,24,32,40,48,56,64,80,96,112,128,144,160,0),
							 "Layer III"=> array(0,8,16,24,32,40,48,56,64,80,96,112,128,144,160,0)
				 )
		   );
	var $sample_tbl =
	 array("MPEG 1" =>array(44100,48000,32000),
		   "MPEG 2" =>array(22050,24000,8000),
		   "MPEG 2.5"=>array(11025,12000,8000)
		 );
	var $channel_tbl=array("Stereo","Joint stereo (Stereo)",
							"Dual channel (Stereo)","Single channel (Mono)");

	
	// read info from filename
	// returns 0 in case of error
	// $this->errormsg is set to a meaningful error
	function read($filename)
	{
		//$this->genrename = getID3GenreList();
		$this->filename=$filename;
		if (! $this->filesize=filesize($filename)){
			$this->errormsg= "MP3Info Error: Could not get filesize for $filename";
			return 0;
		}
		// mp3's are normally bigger than 500 bytes
		if ($this->filesize < 500 ){
			$this->errormsg= "MP3Info Error: $filename to small ($this->filesize) bytes";
			return 0;
		}
		if (! $this->fh=fopen($filename,"rb")){
			$this->errormsg= "MP3Info Error: Could not open file $filename";
			return 0;
		}
		$this->parse_filename(basename($filename));
		$this->tag=$this->readid3v1();
		if (! $this->getmp3info()){
			if ($this->errormsg != ''){
				echo "$this->errormsg<br>\n";
			}
		}
		return 1;

	}
		
    // try to extract information from the filename
	function parse_filename($name)
	{
		// remove .mp3
		$name = substr($name,0,-4);
		$name = str_replace("_20"," ", $name);
		$name = str_replace("_"," ", $name);
		// split name in smaller pieces
		$fields = split(" - ",$name);
		array_walk($fields,'cleanup_field');
		// try to get a track nr
		// must be at the beginning of a field and
		// use: "03.{ Name}" or "3 {Name}" or the field must
		// be just a number :-)
		reset($fields);
		while (list($key,$val)=each($fields)){
			if (ereg("^([0-9]+)\.",$val,$track)){
				$this->track=(int)$track[1];
				$fields[$key]=ereg_replace("^([0-9]+)\.","",$val);
				break;
			}
			if (ereg("^([0-9]+)$",$val,$track)){
				$this->track=(int)$track[1];
				$fields[$key]=ereg_replace("^([0-9]+)$","",$val);
				break;
			}
			if (ereg("^([0-9]+) ",$val,$track)){
				$this->track=(int)$track[1];
				$fields[$key]=ereg_replace("^([0-9]+) ","",$val);
				break;
			}
		}
		// remove emtpy fields
		reset($fields);
		while ( list($key,$val)=each($fields)){
			if (! ereg("^ *$",$val)){
				$fields2[]=$val;
			}
		}
		switch (count ($fields2)){
		case 0:
			// only track number
			break;
		case 1:
			// title
			$this->title= $fields2[0];
	        break;
		case 2:
			// artist - title
			$this->artist=$fields2[0];
		    $this->title=$fields2[1];
	        break;
		case 3:
			// artist - album - title
			$this->artist=$fields2[0];
		    $this->album=$fields2[1];
		    $this->title=$fields2[2];
	        break;
		default:
			echo "Can't interpret tagless file, more than 3 fields in this filename: \"$name\"<br>";
		}
		return 1;
	}


	// try to read a tag file
	function readid3v1()
    {
		// Get the last 128 bytes and see if there's a TAG there
		$seek = fseek($this->fh, $this->filesize - 128);
		$tag = fread($this->fh,128);
		// Grab last char for the Genre
		// This can get confused for EOF with fgets
		$seek = fseek($this->fh, $this->filesize - 1);
		$gen = ord(fgetc($this->fh));
		
		if (strstr("$tag", "TAG") || strstr("$tag", "tag")){
			
			// Extract the tag data
			list($this->title) = explode("\0",substr($tag,3, 30));
			$this->title=chop($this->title);
			list($this->artist) = explode("\0",substr($tag,33, 30));
			$this->artist=chop($this->artist);
			list($this->album) = explode("\0",substr($tag,63, 30));
			$this->album=chop($this->album);
			list($this->year) = explode("\0",str_replace("_","",substr($tag,93, 4)));
			$this->year=chop($this->year);
			list($this->comment) = explode("\0",substr($tag,97,30));
			$this->comment=chop($this->comment);
			// check for id3v1 1.1 tag
			if (ord($tag[125]) == 0){
				// second last comment field was null,
				// so this is maybe a tracknum
				if (ord($tag[126])){
					$this->track = ord($tag[126]);
				}
			}
			$this->genre = $this->genrename[$gen];
			return 1;
		}else{
			return 0;
		}
	}

    // Read general information about file (playtime,mode,bitrate)
	function getmp3info()
    {
		$seek = fseek($this->fh,0);
		// id3v2 tags will be on the beginning.
		// seek to the first frame header (0xFFF)
		$this->seek_frame_header();
		// don't use id3v2 tags during playlength calculation
		// (maybe a pic of the artist is included there ;-)
		$real_size= $this->filesize - ftell($this->fh);
		if ($this->tag){
			$real_size - 128;
		}
		// read in full frame (with xing header)
		$frame=fread($this->fh,32+4+120);
		$b[3]=ord(substr($frame,0,1));
		$b[2]=ord(substr($frame,1,1));
		$b[1]=ord(substr($frame,2,1));
		$b[0]=ord(substr($frame,3,1));
		$mpg=$this->mpeg= $this->mpeg_tbl[( ($b[2]>>3) & 0x3 )];
		$this->layer= $this->layer_tbl[( ($b[2]>>1) & 0x3 )];
		if ($mpg=="MPEG 2.5"){
			$mpg="MPEG 2";
		}
		$this->bitrate= $this->bitrate_tbl[$mpg][$this->layer][( ($b[1]>>4) & 0xF )];
		$this->samplerate= $this->sample_tbl[$this->mpeg][( ($b[1]>>2) & 0x3 )];
		$this->channel = $this->channel_tbl[(($b[0] >> 6) & 0x3)];
		$this->copyright=($b[0] >> 3) & 0x1;
		$this->original=($b[0] >> 2) & 0x1;

		// check for Xing VBR header
        // determine offset of header
		if( $this->mpeg == "MPEG 1" ){
			if( $this->channel == $channel_tbl[3] ){
				// mono
				$vbr=17+4;
			}else{
				$vbr=32+4;
			}
		}else {
			if( $this->channel == $channel_tbl[3] ){
				// mono
				$vbr=9+4;
			}else{
				$vbr=17+4;
			}
		}
		$xing=substr($frame,$vbr,120);
		if (strstr($xing,"Xing")){
			$flags= ord(substr($xing,7,1));
			if ($flags & 0x1){
				$this->vbr=1;
				// we have a framecount
				$tot_frames  = ord(substr($xing,8,1)) * 16777216;
				$tot_frames += ord(substr($xing,9,1)) * 65536;
				$tot_frames += ord(substr($xing,10,1)) * 256;
				$tot_frames += ord(substr($xing,11,1));
				// recalculate real (only frames) filesize
				$real_size= $this->filesize - ftell($this->fh);
				if ($this->tag){
					$real_size - 128;
				}
				$avg_size=$real_size/$tot_frames;
				// calculate the average bitrate of vbr mp3's
				if ($this->mpeg == "MPEG 1"){
					$this->bitrate = ($avg_size * $this->samplerate)/(144*1000);
				}else{
					$this->bitrate = ($avg_size * $this->samplerate)/(12*1000);
				}
			}
		}
		if ($this->bitrate == 0){
			$sec=0;
			//$this->errormsg= "MP3Info Error: $this->filename has bitrate set to 0!";
			return 0;
		}
		$this->sec=($real_size*8) /($this->bitrate *1000);
		return 1;
	}
	
	// seek to the next frame header (0xFFE)
	// $this->fh points to the next filedir
	function seek_frame_header()
	{
		$bytes=0;
		$matched=0;
		while(1){
			$c=fread($this->fh,1);
			if ($bytes++ == 300000){
				// if there is no header after 300 kB this is no mp3 file!
				return 0;
			}
			if ($matched && (ord($c) & 0xE0) == 0xE0){
				// seek back to beginning of frame
				fseek($this->fh, ftell($this->fh) -2);
				break;
			}else{
				$matched=0;
			}
			if (! $matched && (ord($c) == 0xFF)){
				$matched=1;
			}else{
				$matched=0;
			}
		}
		return 1;
	}

	function print_hex($var){
		$len=strlen($var);
		for($i=0;$i<$len; $i++){
			printf ("%s ",ord(substr($var,$i,1)));
		}
		echo "<br>\n";
	}
}

// remove starting and trailing space
// remove crap from splited fields: ({[\w FIELD \w]})
// TODO: Hmm, we need a non greedy \\1
// TODO: should use perl regular expressions...
function cleanup_field(&$field)
{
	$field = preg_replace("/^\s*[\(\{\[\s]*(.*?)[\)\}\]\s]*\s*$/","\\1",$field);
}

?>
