<?
// Functions to draw the result tables of a $db query.

if (!$included["query.inc"])
{
    /** Create a querystring for the given type.
     *
     *  Generates the queries, and executes them on the given DB_Sql Object.
     *  Computes and returns an array with stats about the query.
     *
     *  @param $type   at the moment: "artist", "album", "title", "genre"
     *  @param $by     at the moment: "random", "alpha", "genre", "year", "search" (to search only),
	 *     			                  "hit", "first_seen"
     *  @param $where  The where part of the query
     *  @param $page   (opt) The page to display (defaults to 1)
     *
     *  @return an array containing the stats
     */
    function executeQuery(&$db,$type, $by, $where, $page=1)
    {
        global $dbconfig, $config;

        switch ($type){
        case "artist":
            $q="SELECT artist, loc_id, name, SUM(hits) AS hits, COUNT(*) AS titles,".
               "  SUM(sec) AS sec, type, available ".
               ($by == "random" ? ", (0*loc_id)+rand() AS random " : "") .
               "FROM mp3data m, mp3location l WHERE m.loc_id = l.id AND $where ".
               "GROUP BY m.artist, m.loc_id ".
               "ORDER BY ".
		           ($by == "hit" ? "hits DESC, " : "") .
		           ($by == "random" ? "random ," : "") .
                   " available DESC ". ($by == "search" || $by =="alpha" ||$by == "hit" ? "" : ", $by" );
            break;
        case "album":
            $q="SELECT album,artist, loc_id, name, SUM(hits)/COUNT(*) AS hits, COUNT(*) AS titles, ".
               " SUM(sec) AS sec, type, available ".
                 ($by == "random" ? ", (0*loc_id)+rand() AS random " : "") .
               "FROM mp3data m, mp3location l WHERE m.loc_id = l.id AND $where ".
               "GROUP BY m.album, m.loc_id ".
               "ORDER BY  ".($by == "hit" ? "hits DESC," : "") .
 		   		            ($by == "random" ? "random ," : "") .
                  " available DESC ". ($by == "search"  || $by =="alpha" || $by == "hit" ? "" : ", $by"). ", m.artist";
            break;
        case "title":
        case "genre":
            $q="SELECT m.*, l.name, m.loc_id, type,  available ".
                 ($by == "random" ? ", (0*loc_id)+rand() AS random " : "" ) .
               "FROM mp3data m, mp3location l WHERE m.loc_id = l.id AND $where ".
               "ORDER BY  ".($by == "hit" ? "hits DESC , " : "") .
		                    ($by == "random" ? "random ," : "") .
               " available DESC ". ( ($by == "search"  || $by =="alpha" || $by == "hit") ? "" : ", $by "). ", artist, album, track";
               break;
        default:
            echo "<H1>Error, unknown query Type!</H1><br>\n";
        }
        $count_q = "SELECT count(*) AS sum,sum(size) AS size ,sum(sec) AS sec FROM mp3data m WHERE $where";
        $db->query($count_q);
        $db->next_record();
        $ret["sum"] = $db->Record["sum"];
        $ret["size"] = $db->Record["size"];
        $ret["sec"] = $db->Record["sec"] % 60;
        $ret["min"] = floor($db->Record["sec"]/60);
        $ret["hour"] = floor($ret["min"]/60);
        $ret["min"] = $ret["min"] % 60;
        $ret["min"] = ($ret["min"]<10? "0" : "") . $ret["min"];
        $ret["sec"] = ($ret["sec"]<10? "0" : "") . $ret["sec"];

        $count_q= eregi_replace("^SELECT .* FROM", "SELECT 1 FROM",$q);
        $count_q= eregi_replace("ORDER BY.*$", "",$count_q);
        $db->query($count_q);
        $ret["matched"]=$db->affected_rows();
        if ($dbconfig["sqlserver"] == "mysql")
            $q.=" LIMIT ".(($page-1)*$config["num_per_page"]).",".($config["num_per_page"]);
        elseif ($dbconfig["sqlserver"] == "pgsql")
            $q="  LIMIT ".($config["num_per_page"]).",".($config["num_per_page"]*($page-1));

        $db->query($q);
        return $ret;
    }

    /** print a table of all songs within the current $db query.
     *  @param $db           (req) - the database object to use,
     *                        needed fields:           title/genre  artist/album
     *                           - id                       X
     *                           - artist                   X               X
     *                           - album                    X               X
     *                           - track                    X               X
     *                           - title                    X
     *                           - sec                      X             (sum)
     *                           - year                     X
     *                           - genre                    X
     *                           - hits                     X             (sum)
     *                           - comment (of song)        X
     *                           - loc_id                   X               X
     *                           - name (of location)       X
	 *                           - type (of location)       X               X
	 *							 - available				X				X
     *							one may need more fields, for the optlink parameter
	 *
     *  @param $search       (req) - field that was searched on
     *  @param $page         (req) - nr of this page
     *  @param $highlight    (req) - highlight this field (eg. is the searchterm)
     *  @param $optlink      (req) - an hash array that is appended to the play url. key=>value.
	 * TODO:  @param $add_fields   (opt) - additional field to display
     */
    function display_results(&$db,$search,$page=1,$highlight,$optlink,$add_fields="")
    {
        global $theme, $config, $img,$perm,$auth,$sess,$user, $playlist_id;
		$play_perm=$perm->have_perm("play");
		$playlist_perm=$perm->have_perm("playlist");
		$download_perm=$perm->have_perm("download");
		$edit_id3_perm=$perm->have_perm("edit_id3");
        $search=str_replace("exact","",$search);
        echo "<form action=".$config["index"]." method=post>\n";
        echo "<input type=hidden name=action value=submit>\n";
        $markbgcolor="bgcolor=\"".$theme->highlightcolor."\"";
		// these fields have one row more.
		if ($search == "year" || $search == "genre" || $search == "first_seen"){
			$nr_rows = 5;
		}else{
			$nr_rows = 4;
		}
		if ($playlist_id && playlist_perm){
			$db2 = new rimpsDB;
			$db2->query("select song_id from playlist_detail where playlist_id = " . $playlist_id);
			while ($db2->next_record())
                $playlist_arr[] = $db2->Record["song_id"];
		}
        if ($search=="title" || $search == "genre"){
            echo "<table border=0 cellpadding=1 cellspacing=1>\n";
			echo "  <tr bgcolor=\"".$theme->color2."\">\n";
            //echo "<table border=0 cellpadding=1 cellspacing=1 width=".($config["site_width"]/1.5)."> <tr bgcolor=\"".$theme->color2."\">";
            if ($playlist_id && $playlist_perm)
                //**Add Button***
                echo "    <td rowspan=$nr_rows><input type=image src=\"".$img["add"] ."\" border=0 align=absmiddle name=add height=29 width=40></td>\n";
            else
                echo "    <td rowspan=$nr_rows>&nbsp;</td>\n";
            echo "    <td>Artist</td>\n";
            echo "    <td>Album</td>\n";
            echo "  </tr>\n";
            echo "  <tr bgcolor=\"".$theme->color2."\">\n";
            echo "    <td>Track. Title</td>\n";
            echo "    <td>Length</td>\n";
            echo "  </tr>\n";
            echo "  <tr bgcolor=\"".$theme->color2."\">\n";
            echo "    <td>Location</td>\n";
            echo "    <td>Hits</td>\n";
            echo "  </tr>\n";
            echo "  <tr bgcolor=\"".$theme->color2."\">\n";
            echo "    <td colspan=2><i>Comment</i></td>\n";
            echo "  </tr>\n";
            if ($nr_rows == 5){
                echo "  <tr bgcolor=\"".$theme->color2."\">\n";
                if ($search == "year")
                    echo "    <td>Year</td>\n";
                if ($search == "genre")
                    echo "    <td>Genre</td>\n";
                if ($search == "first_seen")
                    echo "    <td>Added</td>\n";
                echo "  </tr>\n";
            }
            for($i=0; $db->next_record(); $i++){
                // define colors for this entry
                $bgcolor=($i % 2 ? " bgcolor=\"".$theme->tablebgcolor1."\"" : " bgcolor=\"".$theme->tablebgcolor2."\"");
				$min=floor($db->f("sec")/60);
				$sec=floor($db->f("sec")%60);
                echo "  <tr $bgcolor>\n";
                $base=$config["index"]."?action=search";
                if(!empty($db->Record["comment"]))
                    echo "    <td rowspan=".$nr_rows .">";
                else
                    echo "    <td rowspan=". ($nr_rows-1) .">";
				echo ($db->f("available") ? "<IMG src=\"".$img["online"] ."\" alt=\"up\">" :"<IMG src=\"".$img["offline"]."\" alt=\"down\">") . " ";
                if ($playlist_id && $playlist_perm){
                    if (in_array($db->Record["id"], $playlist_arr)){
                        echo "    <td><b><font size=\"+1\">*</font></b>";
                    } else {
                        echo "    <td><input type=checkbox name=" . $db->Record["id"] . ">";
                    }
				}
				if ($play_perm){
					echo "&nbsp;<a href=\"".$config["play"].$GLOBALS['config']['playlist_ext']."?id=".
						$db->Record["id"];
					reset ($optlink);
					while (list($key, $val) = each($optlink)) {
						echo "&".rawurlencode($key) ."=". rawurlencode($val);
					}
					echo "\"><img border=\"0\" alt=\"Play\" src=\"".$img["play"]. "\"></a>";
				}
				if ($download_perm){
					echo "&nbsp;<a href=\"".$config["download"]."?id=".
						$db->Record["id"];
					reset ($optlink);
					while (list($key, $val) = each($optlink)) {
						echo "&".rawurlencode($key) ."=". rawurlencode($val);
					}
					echo "\"><img border=\"0\" alt=\"Download\" src=\"".$img["download"]. "\"></a>";
				}
                if ($edit_id3_perm && $db->f("type") != "smb"){
					# TODO: modify id3_edit to use Mp3Info, and extend MP3Info
					# to allow saving of a file
                    //createLink("id3_edit", "Edit", $browse,$sort,$args="&id=".$db->Record["id"]);
					echo " Edit";
                }
				
                echo " &nbsp;</td>\n";
                echo "    <td ".($highlight == "artist" ? $markbgcolor : $bgcolor).">&nbsp;<a href=\"$base&display=exact_artist&search=".rawurlencode($db->Record["artist"])."\">".$db->Record["artist"]."</a>&nbsp;</td>\n";
                echo "    <td ".(($highlight == "album") ? $markbgcolor : $bgcolor).">&nbsp;<a href=\"$base&display=exact_album&search=".rawurlencode($db->Record["album"])."\">".$db->Record["album"]."</a>&nbsp;</td>\n";
                echo "  </tr>\n";
                echo "  <tr ".$bgcolor.">\n";
                echo "    <td ". ($highlight == "title" ? $markbgcolor : $bgcolor).">&nbsp;<a href=\"$base&display=exact_title&search=".rawurlencode($db->Record["title"])."\">";
                if (!empty($db->Record["track"]) && $db->Record["track"] !=0 ){
                    echo $db->Record["track"] .". ";
                }
                echo $db->Record["title"]."</a>&nbsp;</td>\n";
                echo "    <td align=\"right\">". $min .":". ($sec<10 ? "0" : "") . $sec . "</td>\n";

                echo "  </tr>\n";


                echo "  <tr $bgcolor>\n";
                echo "    <td ".($highlight == "location" ? $markbgcolor : $bgcolor).">&nbsp;<a href=\"$base&display=exact_loc_id&search=".rawurlencode($db->Record["loc_id"])."\">".$db->Record["name"]."</a>&nbsp;</td>\n";
                echo "    <td align=\"right\">&nbsp;" .$db->Record[hits] ."</td>\n";
                echo "  </tr>\n";
                if(!empty($db->Record["comment"])){
                    echo "  <tr $bgcolor>\n";
                    echo "    <td colspan=2>&nbsp;<a href=\"$base&display=exact_comment&search=".rawurlencode($db->Record["comment"])."\"><i>".$db->Record["comment"]."</i></a>&nbsp;</td>\n";
                    echo "  </tr>\n";
                }

                if ($search == "year" || $search == "genre" || $search == "first_seen") {
                    echo "  <tr $bgcolor>\n";
                    if ($search == "year")
                        echo "    <td ".($highlight == "year" ? $markbgcolor : $bgcolor).">".$db->Record["year"]."</td>\n";
                    if ($search == "genre")
                        echo "    <td ".($highlight == "genre" ? $markbgcolor : $bgcolor).">".$db->Record["genre"]."</td>\n";
                    if ($search == "first_seen")
                        echo "    <td ".($highlight == "genre" ? $markbgcolor : $bgcolor).">".$db->Record["genre"]."</td>\n";
                    echo "  </tr>\n";
                }
            }
            echo "</table>\n";

        } else if ($search=="artist" || $search == "album"){
            //      echo "<table border=0 cellpadding=0 cellspacing=1 width=".($config["site_width"]/1.5)."> <tr bgcolor=\"".$theme->color2."\">";
            echo "<table border=0 cellpadding=0 cellspacing=1>";
			echo "  <tr bgcolor=\"".$theme->color2."\">\n";
            echo "    <td>&nbsp;</td>\n";
            echo "    <td align=\"right\">Nr.</td>\n";
            echo "    <td>Artist</td>\n";
            if($search=="album")
                echo "    <td>Album</td>\n";
            echo "    <td align=\"right\">Titles\n</td>\n";
            echo "    <td>Playtime</td>\n";
            echo "    <td>Location</td>\n";
            echo "    <td align=\"right\">Hits\n</td>\n";
            echo "  </tr>\n\n";

            //print the table rows
            for($i=(($page-1)*$config["num_per_page"])+1; $db->next_record(); $i++) {
                // define colors for this entry
                $bgcolor=($i % 2 ? " bgcolor=\"".$theme->tablebgcolor1."\"" : " bgcolor=\"".$theme->tablebgcolor2."\"");
                $sec = floor($db->f("sec") % 60);
                $min = floor($db->f("sec") / 60);

                echo "  <tr $bgcolor>\n";
                echo "    <td>";
				echo ($db->f("available") ? "<IMG src=\"".$img["online"] ."\" alt=\"up\">" :"<IMG src=\"".$img["offline"]."\" alt=\"down\">") . " " ;

				if ($play_perm){
					echo "<a href=\"".$config["play"].$GLOBALS['config']['playlist_ext'].
						"?$search=".rawurlencode($db->Record[$search]).
						"&loc=".$db->Record["loc_id"];
					reset ($optlink);
					while (list($key, $val) = each($optlink)) {
						echo "&".rawurlencode($key) ."=". rawurlencode($val);
					}
					echo "\"><img border=\"0\" alt=\"Play\" src=\"".$img["play"]. "\"></a>";
//					echo "\">Play</a>";
				}
				
				echo "    </td>\n";
				
                $base=$config["index"]."?action=search";
                echo "    <td align=\"right\">$i</td>\n\n";
                echo "    <td ".($highlight == "artist" ? $markbgcolor : $bgcolor).">&nbsp;<a href=\"$base&display=exact_artist&search=".
                    rawurlencode($db->Record["artist"])."\">".$db->Record["artist"].
                    "</a>&nbsp;</td>\n";
                if ($search=="album")
                    echo "    <td ".($highlight == "album" ? $markbgcolor : $bgcolor).">&nbsp;<a href=\"$base&display=exact_album&search=".
                        rawurlencode($db->Record["album"])."\">".$db->Record["album"].
                        "</a>&nbsp;</td>\n";
                echo "    <td align=\"right\"  ".($highlight == "location" ? $markbgcolor : $bgcolor).">".$db->Record["titles"] ."</td>\n\n";
                echo "    <td align=\"right\"> ". ($min<10? "0" : "") . $min . ":". ($sec<10? "0" : "") . $sec ."</td>\n";
                echo "    <td align=\"right\"  ".($highlight == "name" ? $markbgcolor : $bgcolor).">".$db->Record["name"] ."</td>\n";
                echo "    <td align=\"right\" ".($highlight == "hits" ? $markbgcolor : $bgcolor).">".$db->Record["hits"] ."</td>\n";
                echo "  </tr>\n";

                if ($display == "year" or $display == "genre"){
                    echo "  <tr $bgcolor>\n";
                    if ($display == "year")
                        echo "    <td  ".($highlight == "year" ? $markbgcolor : $bgcolor).">".$db->Record["year"]."</td>\n";
                    if ($display == "genre")
                        echo "    <td ".($highlight == "genre" ? $markbgcolor : $bgcolor).">".$db->Record["genre"]."</td>\n";
                    echo "  </tr>\n";
                }
            }
            echo "</table>";
        }
        echo "</form>";
	}
    $included["query.inc"]=1;
}
