<?

function readVorbis($filename)  {

        $fd = fopen($filename, "r");

        if (vorbis_trackdata($fd, $trackdata))
	{
		// fields not caught in header
		$trackdata[size] = filesize($filename);
	}
	else {
		echo "ERROR: failed to process $filename<br>";
	}

        fclose($fd);

        return($trackdata);
}

//
// BUGS
//   does not set track time
//
function vorbis_trackdata($fd, &$trackdata)
{
	// shrug
	$trackdata[mpeg] = 'Ogg-Vorbis';
	$trackdata[layer] = 'Ogg-Vorbis';

	// info packet
	if (!vorbis_read_packet($fd, $packet)) return FALSE;
	if (!vorbis_unpack_info($packet, $info)) return FALSE;

	$trackdata[samplerate] = $info[rate];
	$trackdata[bitrate] = $info[bitrate_nominal] / 1000;
	switch ($info[channels])
	{
		case 2: $trackdata[channel] = 'Stereo'; break;
		case 1: $trackdata[channel] = 'Mono'; break;
		default: $trackdata[channel] = $info[channels];
	}

	// comment packet
	if (!vorbis_read_packet($fd, $packet)) return FALSE;
	if (!vorbis_unpack_comments($packet, $comments)) return FALSE;

	$trackdata[title] = $comments[title];
	$trackdata[artist] = $comments[artist];
	$trackdata[album] = $comments[album];
	$trackdata[year] = substr($comments[date],0,4);
	$trackdata[comment] = $comments[comment];
	$trackdata[genre] = $comments[genre];
	$trackdata[track] = $comments[tracknumber];

	// FIXME
	$trackdata[sec] = 300;

	return TRUE;
}

//
// BUGS
//   none
//
function vorbis_unpack_info($packet, &$info)
{
	$pos = 0;

	if (Ord(substr($packet,$pos,1)) != 1)
		return FALSE;
	$pos += 1;

	if (substr($packet,$pos,6) != "vorbis")
		return FALSE;
	$pos += 6;

	// not sure what's supposed to be at bytes 8-11,
	// is set to zero in lib/info.c
	$pos += 4;

	$info[channels] = Ord(substr($packet, $pos, 1));
	$pos += 1;

	$a = unpack("Vrate/Vbitrate_upper/Vbitrate_nominal/Vbitrate_lower",
	             substr($packet, $pos, 16));
	$info[rate] = $a[rate];
	$info[bitrate_upper] = $a[bitrate_upper];
	$info[bitrate_nominal] = $a[bitrate_nominal];
	$info[bitrate_lower] = $a[bitrate_lower];
	$pos += 16;

	return TRUE;
}

//
// Doesn't do anything yet. Eventually should read track time, at a minimum.
//
function vorbis_unpack_books($packet, &$time)
{
	$pos = 0;

	if (Ord(substr($packet,$pos,1)) != 5)
		return FALSE;
	$pos += 1;

	if (substr($packet,$pos,6) != "vorbis")
		return FALSE;
	$pos += 6;

	return TRUE;
}

//
// BUGS
//    vorbis comment header field names are not required to be unique.
//       This function will only use the last field seen in the case
//       of duplicate field names.
//
function vorbis_unpack_comments($packet, &$comments)
{
	$pos = 0;

	if (Ord(substr($packet,$pos,1)) != 3)
		return FALSE;
	$pos += 1;

	if (substr($packet,$pos,6) != "vorbis")
		return FALSE;
	$pos += 6;

	$a = unpack("Vvendor_length", substr($packet, $pos, 4));
	$pos += 4 + $a["vendor_length"];
	$a = unpack("Vnum_comments", substr($packet, $pos, 4));
	$pos += 4;

	for ($i=0; $i<$a[num_comments]; $i++) {

		$b = unpack("Vcomment_length", substr($packet, $pos, 4));
		$pos += 4;

		$this_comment = substr($packet, $pos, $b[comment_length]);
		$pos += $b["comment_length"];

		list($field, $val) = explode("=", $this_comment, 2);
		$comments[strtolower($field)] = $val;
	}

	return TRUE;
}

//
// BUGS
//   see ogg_read_page()
//
function vorbis_read_packet($fd, &$packet_data)
{
	$complete = FALSE;
	$packet_data = "";
	while (!$complete) {
		if (!ogg_read_page($fd, $tmp, $complete))
			return FALSE;
		$packet_data .= $tmp;
	}

	return TRUE;
}

//
// BUGS
//   no checking of CRC
//   does not account for packet starting or stopping mid-page
//
function ogg_read_page($fd, &$page_content, &$complete)
{

	$header = fread($fd, 27);
	if (strlen($header) != 27) return FALSE;

	$capture_pattern = substr($header, 0, 4);
	if ($capture_pattern != "OggS") return FALSE;

	// PCM absolute position is a 64-bit value, -1 if no packets
	// finish on current page
	$position = unpack("Vlow/Vhigh", substr($header, 6, 8));
	if ($position["low"] == -1 && $position["high"] == -1)
		$complete = FALSE;
	else
		$complete = TRUE;

	$page_segments = Ord(substr($header, 26, 1));

	$lacing_values = fread($fd, $page_segments);
	if (strlen($lacing_values) != $page_segments) return FALSE;

	$page_content = "";
	for ($i=0; $i<strlen($lacing_values); $i++) {
		$length = Ord(substr($lacing_values, $i, 1));
		$tmp = fread($fd, $length);
		if (strlen($tmp) != $length) return FALSE;
		$page_content  .= $tmp;
	}

	return TRUE;
}

?>
